# -*- coding: UTF-8 -*-
"""
Blog-app: First Python-Flask webapp to discuss flask related topics
"""
from flask import Flask,render_template,request,flash,redirect,abort, url_for
from markupsafe import escape
import sqlite3

def get_db_connection():
    conn = sqlite3.connect('database.db')
    conn.row_factory = sqlite3.Row
    return conn


app = Flask(__name__)    # Construct an instance of Flask class for our webapp


app.config['SECRET_KEY'] = 'flaskexample2023'


@app.route('/')   # URL '/' to be handled by main() route handler (or view function)
#@app.route('/post')
#@app.route('/hi')
def main():
    """ Render an HTML template and return"""
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close() 
    return render_template('blog.html', posts=posts) #HTML file to be placed under sub-directory templates

@app.route('/home')
def home():
     return redirect(url_for('main')) #url_for('main'): returns the internal (relative) URL '/'


@app.route('/login')
def login():
    abort(401)
    this_is_never_executed()
	
@app.errorhandler(404)
def page_not_found(error):
    return render_template('page_not_found.html'), 404

@app.route('/author/<authorname>')
def show_author_name(authorname):
    conn = get_db_connection()
    posts = conn.execute('SELECT * FROM posts').fetchall()
    conn.close()
    return render_template('blog.html', posts=posts,author=f'{escape(authorname)}') 

def get_post(post_id):
    conn = get_db_connection()
    post = conn.execute('SELECT * FROM posts WHERE id = ?',(post_id,)).fetchone()
    conn.close()
    if post is None:
        abort(404)
    return post

@app.route('/post/<int:post_id>')
def post(post_id):
    post = get_post(post_id)
    return render_template('post.html', post=post)


@app.route('/create', methods=('GET', 'POST'))
def create():
    if request.method == 'POST':
        title = request.form['title']
        content = request.form['content']

        if not title:
            flash('Title is required!')
        else:
            conn = get_db_connection()
            conn.execute('INSERT INTO posts (title, content) VALUES (?, ?)',
                         (title, content))
            conn.commit()
            conn.close()
            return redirect(url_for('home'))

    return render_template('create.html')


@app.route('/<int:id>/edit', methods=('GET', 'POST'))
def edit(id):
    post = get_post(id)

    if request.method == 'POST':
        title = request.form['title']
        content = request.form['content']

        if not title:
            flash('Title is required!')
        else:
            conn = get_db_connection()
            conn.execute('UPDATE posts SET title = ?, content = ?'
                         ' WHERE id = ?',
                         (title, content, id))
            conn.commit()
            conn.close()
            return redirect(url_for('main'))

    return render_template('edit.html', post=post)


@app.route('/<int:id>/delete', methods=('POST',))
def delete(id):
    post = get_post(id)
    conn = get_db_connection()
    conn.execute('DELETE FROM posts WHERE id = ?', (id,))
    conn.commit()
    conn.close()
    flash('"{}" was successfully deleted!'.format(post['title']))
    return redirect(url_for('home'))








if __name__ == '__main__':  
    app.run(debug=True) # Enable reloader and debugger and Launch built-in web server and run this Flask webapp
